@@ -68,8 +68,9 @@ predict.timereg <-function(object,newdata=NULL,X=NULL,times=NULL,
 {
     ## {{{
 ###    if (object$conv$convd>=1) stop("Model did not converge.")
-    if (!(inherits(object,'comprisk') || inherits(object,'aalen')
-          || inherits(object,'cox.aalen')))
+ ### {{{ reading designs  and models 
+  if (!(inherits(object,'comprisk') || inherits(object,'aalen')
+        || inherits(object,'cox.aalen')))
         stop ("Must be output from comp.risk function")
 
   if(inherits(object,'aalen')) { modelType <- 'aalen';
@@ -89,13 +90,17 @@ predict.timereg <-function(object,newdata=NULL,X=NULL,times=NULL,
     stop("resample processes necessary for these computations, set resample.iid=1");
   }
   if (is.null(object$gamma)==TRUE) { semi<-FALSE } else { semi<-TRUE }
+  ## }}} 
 
   ## {{{ extracts design based on the different specifications
   ## cox.aalen uses prop(...), while aalen and comp.risk use const(...)
   ## this accounts for the different number of characters
   if(inherits(object,'cox.aalen')){ indexOfFirstChar <- 6; } else { indexOfFirstChar <- 7; }
   
-  if (!is.null(newdata)) { 
+  ### whether or not iid time coarsening is used (only for cox-aalen)
+  ### or whether time is changed due to times argument, then also changing times for iid and cum
+  iidtimechange <- 0; iidtime <- 0
+  if (!is.null(newdata)) {  ## {{{ newdata given 
     ##  The time-constant effects first
     formulao <- attr(object,"Formula")
     des <- aalen.des2(formula(delete.response(terms(formulao))),data=newdata,model=modelType)
@@ -109,14 +114,14 @@ predict.timereg <-function(object,newdata=NULL,X=NULL,times=NULL,
     ## Then extract the time-varying effects
     ###    time.coef <- data.frame(object$cum)
     time.coef <- as.matrix(object$cum)
-    if (!is.null(times)) time.coef <- Cpred(time.coef,times)
+    if (!is.null(times)) {time.coef<-Cpred(time.coef,times); iidtimechange <- 1; iidtime <- object$cum[,1];} 
     ### SE based on iid decomposition so uses time-resolution for cox.aalen model 
-    if (modelType=="cox.aalen" && (!is.null(object$time.sim.resolution)) && (se==TRUE)) time.coef <- Cpred(object$cum,object$time.sim.resolution)
-    ntime <- nrow(time.coef)
-    fittime <- time.coef[,1,drop=TRUE]
-    ntimevars <- ncol(time.coef)-2
-    nobs <- nrow(newdata)
-  } else if ((is.null(Z)==FALSE) || (is.null(X)==FALSE)){ 
+    if (modelType=="cox.aalen" && (!is.null(object$time.sim.resolution)) && (se==TRUE)) 
+    { iidtime <- object$time.sim.resolution; iidtimechange <- 1} 
+
+    nobs <- nrow(newdata) 
+    ## }}} 
+  } else if ((is.null(Z)==FALSE) || (is.null(X)==FALSE)){ ## {{{ X, Z specified  
 
     if (semi) zcol <- length(c(object$gamma)) else zcol <- NULL
     if (!is.null(Z)) { prow <- nrow(Z); } 
@@ -130,25 +135,30 @@ predict.timereg <-function(object,newdata=NULL,X=NULL,times=NULL,
 	    X[,1] <- 1
     }
     if (semi & is.null(Z)) Z <- matrix(0,nrow=nrow(X),ncol=zcol); 
-
     time.vars <- X
     if (semi) constant.covs <- Z else constant.covs <- NULL
 
-    nobs<-nrow(time.vars);
-    ## extract the time-varying effects
+    nobs<-nrow(X);
+
+    ## Then extract the time-varying effects
     time.coef <- as.matrix(object$cum)
-    if (modelType=="cox.aalen" && (!is.null(object$time.sim.resolution))) time.coef <- Cpred(object$cum,object$time.sim.resolution)
-    ntime <- nrow(time.coef)
-  } else {
+
+    if (!is.null(times)) {time.coef<-Cpred(time.coef,times); iidtimechange <- 1; iidtime <- object$cum[,1];} 
+    ### SE based on iid decomposition so uses time-resolution for cox.aalen model 
+    if (modelType=="cox.aalen" && (!is.null(object$time.sim.resolution)) && (se==TRUE)) 
+    { iidtime <- object$time.sim.resolution; iidtimechange <- 1} 
+
+    ## }}} 
+  } else { ## {{{ 
     stop("Must specify either newdata or X, Z\n");
-  }
+  } ## }}} 
 
   ## }}}
 
   ## {{{ predictions for competing risks and survival data
 
   cumhaz<-as.matrix(time.vars) %*% t(matrix(time.coef[,-1],ncol=(ncol(time.coef)-1)))
-  time<-time.coef[,1]; 
+  times <- time<-time.coef[,1]; 
   if (semi==TRUE) pg <- nrow(object$gamma); 
   nt<-length(time);
 
@@ -168,7 +178,7 @@ predict.timereg <-function(object,newdata=NULL,X=NULL,times=NULL,
       } else {
         P1=1-exp(-cumhaz-constant.part )
       }
-      RR<-NULL; 
+      RR<-1; 
     } else if (modelType == 'rcif') { # P1=exp(x^T b(t) + z^t t^p gamma) 
         if (semi==FALSE){
            P1=exp(cumhaz);
@@ -178,11 +188,12 @@ predict.timereg <-function(object,newdata=NULL,X=NULL,times=NULL,
        RR<-1;
     } else if (modelType == 'rcif2') { # P1=x^T b(t) exp( z^t t^p gamma) 
         if (semi==FALSE){
-           P1=cumhaz;
+         P1=cumhaz;
+         RR<-1;
          } else {
          P1<-cumhaz*exp(constant.part);
+         RR<-exp(constant.part);
        }
-       RR<-1;
     } else if (modelType == 'prop') {# model proportional , Fine Gray extension
         if (semi==FALSE){
         RR<-exp(cumhaz);
@@ -201,8 +212,8 @@ predict.timereg <-function(object,newdata=NULL,X=NULL,times=NULL,
       if (semi==FALSE){ RR<-exp(cumhaz); }   else { RR<-exp(cumhaz+constant.part); }
       P1<-RR/(1+RR);
     } else if (modelType == 'logistic2') { #model logistic, baseline-par
-      if (semi==FALSE){ RR<-cumhaz; }   else { RR<-cumhaz*exp(constant.part); }
-      P1<-RR/(1+RR);
+      if (semi==FALSE){ RR<-1; }   else { RR<-exp(constant.part); }
+      P1<-RR*cumhaz/(1+RR*cumhaz);
     } ## }}}
     } 
     else if (modelType=="prop.odds")
@@ -237,7 +248,10 @@ predict.timereg <-function(object,newdata=NULL,X=NULL,times=NULL,
     pg<-length(object$gamma); 
     delta<-c();
     for (i in 1:n) {
-       tmp<- as.matrix(time.vars) %*% t(object$B.iid[[i]]) 
+       if (iidtimechange==1) 
+       tmptiid<- t(Cpred(cbind(iidtime,object$B.iid[[i]]),times)[,-1,drop=FALSE])
+       else tmptiid <- t(object$B.iid[[i]])
+       tmp<- as.matrix(time.vars) %*% tmptiid
 
        if (semi==TRUE) {
              gammai <- matrix(object$gamma.iid[i,],pg,1); 
@@ -260,24 +274,38 @@ predict.timereg <-function(object,newdata=NULL,X=NULL,times=NULL,
       if (semi==TRUE){
         if(modelType=="additive" || modelType == "aalen") { tmp<-tmp+ tmp.const } 
 	else if (modelType=="prop" || modelType=="rcif") { tmp<-RR*tmp+RR*tmp.const; } 
-	else if (modelType=="logistic" || modelType=="rcif2") { tmp<-RR*tmp+RR*tmp.const; } 
-	else if (modelType=="logistic2") { tmp<-RR*tmp+RR*tmp.const; } 
+	else if (modelType=="logistic" || modelType=="rcif2") { tmp<-RR*tmp+RR*cumhaz*tmp.const; } 
+	else if (modelType=="logistic2") { tmp<-RR*tmp+RR*cumhaz*tmp.const; } 
 	else if (modelType=="cox.aalen") { tmp <- RR * tmp + RR * cumhaz * tmp.const }
 	else if (modelType=="prop.odds") { tmp <- RR * tmp + RR * cumhaz * tmp.const; }
+      } else {
+	if (modelType=="prop") { tmp<-RR*tmp; } 
       }
+
       delta<-cbind(delta,c(tmp)); 
     }
     se<-apply(delta^2,1,sum)^.5
 
-    if(modelType == 'additive' || modelType == 'prop' || modelType=="fg"){ se.P1<-matrix(se,nobs,nt)*(1-P1); 
+    if(modelType == 'additive' || modelType == 'prop' || modelType=="fg"){ 
+       se.P1<-matrix(se,nobs,nt)*(1-P1); 
        if (resample.iid==1)  P1.iid <- array(delta*c(1-P1),c(nobs,nt,n));   
     } 
-    else if(modelType == 'rcif' || modelType== 'rcif2'){ se.P1<-matrix(se,nobs,nt)*(P1) 
-       if (resample.iid==1) P1.iid <- array(delta*c(P1),c(nobs,nt,n));   
+    else if(modelType == 'rcif' ){ 
+       se.P1<-matrix(se,nobs,nt)*P1 
+       if (resample.iid==1) P1.iid <- array(delta*P1,c(nobs,nt,n));   
     } 
-    else if (modelType == 'logistic' || modelType == 'logistic2'){ se.P1<-matrix(se,nobs,nt)*P1/(1+RR) 
+    else if(modelType == 'rcif2'){ 
+       se.P1<-matrix(se,nobs,nt) 
+       if (resample.iid==1) P1.iid <- array(delta,c(nobs,nt,n));   
+    }
+    else if (modelType == 'logistic'){ 
+       se.P1<-matrix(se,nobs,nt)*P1/(1+RR) 
        if (resample.iid==1) P1.iid <- array(delta*c(P1/(1+RR),c(nobs,nt,n)));   
     } 
+    else if (modelType == 'logistic2'){ 
+       se.P1<-matrix(se,nobs,nt)*1/(1+cumhaz*RR)^2 
+       if (resample.iid==1) P1.iid <- array(delta*c(1/(1+cumhaz*RR),c(nobs,nt,n)));   
+    } 
     else if (modelType == 'aalen' || modelType == 'cox.aalen'){ 
        se.S0<-matrix(se,nobs,nt)*S0 
        if (resample.iid==1) S0.iid <- array(delta*c(S0),c(nobs,nt,n));   
